
import com.ewon.ewonitf.IOServer;
import com.ewon.ewonitf.IOTag;
import com.ewon.ewonitf.IOValue;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;
import javax.microedition.io.SocketConnection;


  public class IOserver extends IOServer implements Runnable {
    
      //Array of Tags to read
      private Vector TagsList = new Vector();
            
      //USER Params
      private int int_Debug = 0;
      private int int_PollRate = 10000;
      
      SocketConnection sc  = null;
      InputStream is  = null;
      OutputStream os  = null;
      
      
      
    public void onRegister() throws Exception
   {
      // setJavaReplyTimeout(1000000);   //Let Java freeze the eWON while debugging.
   }
      
    public IOserver()
   {
      super();      
      
      //Set IOServer name
      setServerName("WEATHER");      
   }
       
    public void onPassConfig(boolean applyConfig, boolean checkConfig) throws Exception 
    {
        //Read User parameters
        WebClient.ServerIP = getConfigParam("ip","192.168.120.93");      
        int_Debug = Integer.parseInt(getConfigParam("debug","0"));
        
        if (int_Debug == 1) System.out.println("WEATHERSTATION DEBUG: ip = " + WebClient.ServerIP);
        if (int_Debug == 1) System.out.println("WEATHERSTATION DEBUG: debug = " + String.valueOf(int_Debug));
        if (int_Debug == 1) System.out.println("WEATHERSTATION DEBUG: pollrate = " + String.valueOf(int_PollRate));
    }
 
    public IOValue onGetIo(IOTag ioTag) throws Exception
    {
       return ioTag.getIoData();
    }
           
    public IOTag onGetIoInfo(String topicName, String ioName) throws Exception {
      
        IOTag ioTag; 
        
        //All tags are set as Float Tags and initiated with a BAD quality till the first polling happens
        ioTag = new IOTag(topicName, ioName, IOValue.DATATYPE_FLOAT32, true);  
        ioTag.setQuality(IOTag.QUALITY_BAD);
        
        postQuality(ioTag);
               
      return ioTag;
   }


     public void onAdviseIo(IOTag ioTag) throws Exception 
     {        
        TagsList.addElement(new TagObject(ioTag));                   
     }

     public void onUnadviseIo(IOTag ioTag) throws Exception 
     {
        for(int i=0;i<TagsList.size();i++)
        {
            TagObject tag_o = (TagObject) TagsList.elementAt(i);
            
            if (tag_o.ioTag.getIoName().equals(ioTag.getIoName()))
            {
                TagsList.removeElement(tag_o);
                break;
            }
        }
     }
   
     public void onPutIo(IOTag ioTag, IOValue ioData) throws Exception
     {
        //In case of writing
        //No Tag to write to the Wheather Station
     
     }         
           
     public void run()
     {             

         while(true)
         {       
            try 
            {                              
                if(TagsList.isEmpty())
                {
                    if (int_Debug == 1) System.out.println("WEATHERSTATION DEBUG: No tag to process" );
                    Thread.sleep(5000);
                    continue;
                }
                
                if (WebClient.ServerIP == null)
                {
                    if (int_Debug == 1) System.out.println("WEATHERSTATION DEBUG: Waiting for IOServer parameters");
                    Thread.sleep(5000);
                    continue;
                }

                
               //Comm. Get the file over HTTP
               if (int_Debug == 1) System.out.println("WEATHERSTATION DEBUG: Collect Tags (HTTP) - Start");
               WebClient.CollectTags();       
               if (int_Debug == 1) System.out.println("WEATHERSTATION DEBUG: Collect Tags (HTTP) - End");
                       
                
               //Loop on each Tag to read
                for(int i=0;i<TagsList.size();i++)   
                {                                    
                    TagObject Tag_o = (TagObject) TagsList.elementAt(i);
                    
                    //Reset Retry number
                    Tag_o.retryNb = 0;
                    
                    try
                    {
                       String sz_NewValue = Items.getValue(Tag_o.ioTag.getIoName());
                       float NewValue = Float.parseFloat(sz_NewValue);
                       
                       //Update value only if it has changed or when quality is back ok                       
                       if (int_Debug == 1) System.out.println("WEATHERSTATION DEBUG: Current value for Tag " + Tag_o.ioTag.getIoName() + " : " + String.valueOf((Tag_o.ioTag.getValueFloat32())));
                       if (Tag_o.ioTag.getValueFloat32() != NewValue || Tag_o.ioTag.getQuality() == IOTag.QUALITY_BAD)
                       {
                            Tag_o.ioTag.setValueFloat32(NewValue);
                            postIoChange(Tag_o.ioTag);                        
                            if (int_Debug == 1) System.out.println("WEATHERSTATION DEBUG: New value for Tag " + Tag_o.ioTag.getIoName() + " : " + String.valueOf(NewValue));
                       }
                       
                       //Update quality only if it has changed
                       if (Tag_o.ioTag.getQuality() == IOTag.QUALITY_BAD)
                       {
                           Tag_o.ioTag.setQuality(IOTag.QUALITY_GOOD);
                           postQuality(Tag_o.ioTag);    
                           if (int_Debug == 1) System.out.println("WEATHERSTATION DEBUG: New quality for Tag " + Tag_o.ioTag.getIoName() + " : Quality GOOD");
                       }
                       
                    }
                    catch(Exception e)
                    {
                        System.out.println("WEATHERSTATION ERROR: Cannot read value of Tag " + Tag_o.ioTag.getIoName());
                        System.out.println("WEATHERSTATION ERROR: " + e.getMessage());
                        
                        //Set Tag to BAD Quality after 3 retries
                        Tag_o.retryNb ++;
                        if (Tag_o.retryNb > 2)
                        {
                            if (int_Debug == 1) System.out.println("WEATHERSTATION DEBUG: Tag set to BAD quality.");
                            Tag_o.ioTag.setQuality(IOTag.QUALITY_BAD);
                            postQuality(Tag_o.ioTag);                                                          
                        }
                       
                        

                    }                    
                }                                
                
                Thread.sleep(int_PollRate);                
            } 
            catch (Exception ex) 
            {
                 System.out.println("WEATHERSTATION ERROR: " + ex.getMessage());
                 System.out.println("WEATHERSTATION ERROR: Next Try in 30 seconds");
                 
                 //major failure --> Set quality to BAD 
                 try 
                 {
                    for(int i=0;i<TagsList.size();i++)   
                    {      
                         TagObject Tag_o = (TagObject) TagsList.elementAt(i);
                         
                         //Set Tag to BAD Quality after 3 retries
                         Tag_o.retryNb ++;
                         if (Tag_o.retryNb > 2)
                         {
                            if (int_Debug == 1) System.out.println("WEATHERSTATION DEBUG: All Tags set to BAD quality.");
                            Tag_o.ioTag.setQuality(IOTag.QUALITY_BAD);
                            postQuality(Tag_o.ioTag);  
                         }                       

                    }
                     
                        Thread.sleep(30000);
                  } 
                  catch (Exception ex1) 
                  {                     
                     System.out.println("WEATHERSTATION ERROR: Cannot set all tags to BAD, " + ex1.getMessage());
                  }
             }                                     
         }       
     }        
  }
  
    

